@@ -14,6 +14,7 @@ from pydantic import BaseModel, Extra, Field, validator
 
 from .abstract import BaseContent
 from .program import ProgramContent
+from ..exceptions import UnknownHashError
 
 
 class Chain(str, Enum):
@@ -48,6 +49,26 @@ class ItemType(str, Enum):
     storage = "storage"
     ipfs = "ipfs"
 
+    @classmethod
+    def from_hash(cls, item_hash: str) -> 'ItemType':
+        # https://docs.ipfs.io/concepts/content-addressing/#identifier-formats
+        if item_hash.startswith("Qm") and 44 <= len(item_hash) <= 46:  # CIDv0
+            return cls.ipfs
+        elif item_hash.startswith("bafy") and len(item_hash) == 59:  # CIDv1
+            return cls.ipfs
+        elif len(item_hash) == 64:
+            return cls.storage
+        else:
+            raise UnknownHashError(f"Unknown hash {len(item_hash)} {item_hash}")
+
+    @classmethod
+    def is_storage(cls, item_hash: str):
+        return cls.from_hash(item_hash) == cls.storage
+
+    @classmethod
+    def is_ipfs(cls, item_hash: str):
+        return cls.from_hash(item_hash) == cls.ipfs
+
 
 class MongodbId(BaseModel):
     "PyAleph returns an internal MongoDB id"
@@ -147,6 +168,7 @@ class StoreContent(BaseContent):
 class ForgetContent(BaseContent):
     "Content of a FORGET message"
     hashes: List[str]
+    aggregates: List[str] = Field(default_factory=list)
     reason: Optional[str] = None
 
     def __hash__(self):
@@ -195,6 +217,8 @@ class BaseMessage(BaseModel):
     item_hash: str = Field(description="Hash of the content (sha256 by default)")
     content: BaseContent = Field(description="Content of the message, ready to be used")
 
+    forgotten_by: Optional[List[str]]
+
     @validator("item_content")
     def check_item_content(cls, v: Optional[str], values):
         item_type = values["item_type"]
@@ -267,6 +291,12 @@ class ForgetMessage(BaseMessage):
     type: Literal[MessageType.forget]
     content: ForgetContent
 
+    @validator('forgotten_by')
+    def cannot_be_forgotten(cls, v: Optional[List[str]], values) -> Optional[List[str]]:
+        if v:
+            raise ValueError("This type of message may not be forgotten")
+        return v
+
 
 class ProgramMessage(BaseMessage):
     type: Literal[MessageType.program]
